# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['melpoi', 'melpoi.scraping.website-monitor', 'melpoi.sql.SQLAnalyzer']

package_data = \
{'': ['*']}

install_requires = \
['graphviz>=0.20.1,<0.21.0',
 'ipython>=8.6.0,<9.0.0',
 'opencv-python>=4.6.0.66,<5.0.0.0',
 'selenium>=4.6.0,<5.0.0']

setup_kwargs = {
    'name': 'melpoi',
    'version': '0.0.11',
    'description': '',
    'long_description': '# melpoi\nmelpoi is a repository with some of the scripts me myself personally find useful. They can be a simple ETL script or even a web scraping script. This is mainly for me to have a centralized place to look for scripts that I need. Feel free to use them if they can help you, you can also submit a pull request to add new things into this that you feel others might benefit from it.\n\n# Categories\n- [Scraping](https://github.com/la0bing/melpoi/tree/main/melpoi/scraping)\n- [SQL](https://github.com/la0bing/melpoi/tree/main/melpoi/sql)\n',
    'author': 'Melvin Low',
    'author_email': 'la0bing07148@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
